/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: avi_tools.c,v 1.6 2005/08/14 18:26:23 eku Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "avifile.h"
#include "logging.h"

static int
checkavi1(char *h)
{
  if (strncmp(h, "RIFF", 4) != 0)
    return 0;
  if (strncmp(h + 8, "AVI LIST", 8) != 0)
    return 0;
  if (strncmp(h + 20, "hdrlavih", 8) != 0)
    return 0;
  return 1;
}

int
get_avi_info(char *filename, MainAVIHeader * info)
{
  FILE *f;
  char h[32];

  if ((f = fopen(filename, "r")) == NULL)
    return 0;

  fread(h, 32, 1, f);
  if (checkavi1(h) != 1)
    {
      do_log(1, "Warning : Bad AVI File (MainAVIHDR)\n");
      return (0);
    }

  fread(info, sizeof(*info), 1, f);

  do_log(1, "AVI Width is : %lu\n", info->dwWidth);
  do_log(1, "AVI Height is : %lu\n", info->dwHeight);
  do_log(1, "AVI MAXByte : %lu\n", info->dwMaxBytesPerSec);

  fread(h, 20, 1, f);

  while (1)
    {
      if (strncmp(h, "LIST", 4) != 0)
	{
	  do_log(4, "Finished LISTs\n");
	  break;		/* Finished LISTs */
	}
      if (strncmp(h + 8, "strlstrh", 8) == 0)
	{
	  /* We are on a stream header */
	  AVIStreamHeader sth;
	  char fourcc1[5];
	  char fourcc2[5];

	  memset(fourcc1, 0, 5);
	  memset(fourcc2, 0, 5);
	  fread(&sth, sizeof(sth), 1, f);
	  memcpy(fourcc1, &sth.fccType, 4);
	  memcpy(fourcc2, &sth.fccHandler, 4);
	  do_log(1, "Stream Fourcc1=%s\n", fourcc1);
	  do_log(1, "Stream Fourcc2=%s\n", fourcc2);
	  do_log(1, "Rate = %lu\n", sth.dwRate);
	}
      else
	do_log(1, "Unknown list LISTs\n");
      fread(h, 20, 1, f);
    }

  fclose(f);
  return 1;
}

/* EOF */
