/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: cmd_action.c,v 1.29 2005/09/12 11:01:29 eku Exp $
 */

#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "dp500.h"
#include "logging.h"
#include "xmalloc.h"
#if 0
#include "common_menu.h"
#endif
#include "avifile.h"
#ifdef VCDSUPPORT
#include "vcd_tools.h"
#endif
#ifdef DVDSUPPORT
#include "dvd_tools.h"
#endif
#include "playlists.h"
#include "files_tools.h"
#include "shoutcast.h"
#include "cmds.h"


#if 0
static void
action_on_menu(const char *path, t_client * clt, t_dp500 * dp500)
{
  if (clt->action == 1)
    {
      printf("ACT ON MENU\n");
      exec_common_menu_item(path + strlen(MENU_PREFIX), clt);
    }
}
#endif

static void
action_on_stream(char *path, t_client * clt, t_dp500 * dp500)
{
  if (clt->action == 1)
    {
      /* This is a mp3 stream... let's start a thread */
      *(strrchr(path, '.')) = '\0';	/* Strip the ending '.mp3' */
      do_log2(3, clt->fd, "%s: stream '%s'\n", __func__, path);
      clt->stream = shoutcast_open(path, clt);
    }
}

void
cmd_action(char *args, t_client * clt, t_dp500 * dp500)
{
  char *p1, *p2, *p3;
  char *fullpath = NULL, *fext;
  int status = 200;
  char repl[8];

  p1 = strrchr(args, '|');
  p2 = strrchr(args, '\\');
  if (p2 != NULL)
    *p2 = '/';

  /* Parse ACTION command and path */
  clt->action = strtol(args, &p3, 10);
  switch (clt->action)
    {
    case 1:
      fullpath = p1 + 2;
      break;

    case 2:
      fullpath = p3 + 1;
      break;

    default:
      do_log2(2, clt->fd, "%s: invalid ACTION type '%d'\n", __func__,
	      clt->action);
      status = 404;
      goto out;
    }

  do_log2(4, clt->fd, "%s: path '%s'\n", __func__, fullpath);

  /* REPLY status 
     200: OK
     404: Not Found
     ...: others?
   */

  if (strncmp(fullpath, "http://", 7) == 0)
    {
      /* ACTION on a shoutcast stream */
      action_on_stream(fullpath, clt, dp500);
      goto out;
    }
#if 0
  if (strncmp(fullpath, MENU_PREFIX, strlen(MENU_PREFIX)) == 0)
    {
      action_on_menu(fullpath, clt, dp500);
      goto out;
    }
#endif

  /* Handle filename masquarading. Currently this is used for:
     - subpicture files that are not located in same directory as video
     - svcd images, which are extracted to a temporary file */
  do_log2(4, clt->fd, "%s: filename translation: real '%s' requested '%s'\n",
	  __func__, dp500->real_fullpath, dp500->requested_fullpath);

  if ((dp500->requested_fullpath != NULL) && (dp500->real_fullpath != NULL))
    {
      if (strcmp(fullpath, dp500->requested_fullpath) == 0)
	{
	  /* Use alternate path */
	  fullpath = dp500->real_fullpath;
	  do_log2(3, clt->fd,
		  "%s: using alternate file '%s' for requested '%s'\n",
		  __func__, dp500->real_fullpath, dp500->requested_fullpath);
	}
      else
	{
	  do_log2(3, clt->fd,
		  "%s: alternate file set to '%s' does not equal requested file '%s'\n",
		  __func__, dp500->real_fullpath, dp500->requested_fullpath);
	}
    }
  else
    {
      do_log2(3, clt->fd,
	      "%s: filenames not set, real '%s', requested '%s'\n", __func__,
	      dp500->real_fullpath, dp500->requested_fullpath);
    }

  /* Parse filename + path + extension */
  fext = strrchr(fullpath, '.');
  fext++;
  do_log2(4, clt->fd, "%s: sub extension '%s'\n", __func__, fext);

  /* Check file found */
  if (access(fullpath, R_OK) < 0)
    {
      /* Requested file not Found -> try a .bin or scan for subs */
      do_log2(3, clt->fd,
	      "%s: %s (%s) not found, looking for alternatives..\n", __func__,
	      fullpath, fext);
      if (strcasecmp(fext, "mpg") == 0)
	{
#ifdef VCDSUPPORT
	  char *masq_path;
	  char *mpeg_temp_path;

	  /* Are we masquarading a .bin file? */
	  masq_path = xstrdup(fullpath);
	  strcpy(masq_path + strlen(masq_path) - 3, "bin");
	  do_log2(3, clt->fd, "%s: looking for (S)VCD image '%s'\n", __func__,
		  masq_path);
	  if (access(masq_path, R_OK) == 0)
	    {
	      /* Found the masqaraded bin file */
	      strcpy(masq_path + strlen(masq_path) - 3, "mpg");
	      mpeg_temp_path = vcd_to_mpeg_start(masq_path);
	      xfree(dp500->requested_fullpath);
	      dp500->requested_fullpath = xstrdup(fullpath);
	      xfree(dp500->real_fullpath);
	      dp500->real_fullpath = mpeg_temp_path;
	      do_log2(3, clt->fd,
		      "%s: found alternative '%s' for requested '%s'\n",
		      __func__, dp500->real_fullpath,
		      dp500->requested_fullpath);
	    }
#ifdef DVDSUPPORT
	  /* Are we masquarading a DVD title inside a vob file? */
	  else if (strstr(fullpath, "DVDTITLE") != NULL)
	    {
	      /* Found the masqaraded DVD file */
	      strcpy(masq_path + strlen(masq_path) - 3, "mpg");
	      mpeg_temp_path = dvd_to_mpeg_start(masq_path);
	      xfree(dp500->requested_fullpath);
	      dp500->requested_fullpath = xstrdup(fullpath);
	      xfree(dp500->real_fullpath);
	      dp500->real_fullpath = mpeg_temp_path;
	      do_log2(3, clt->fd,
		      "%s: found alternative '%s' for requested '%s'\n",
		      __func__, dp500->real_fullpath,
		      dp500->requested_fullpath);
	    }
#endif
	  else
	    {
	      /* Not found */
	      status = 404;
	      do_log2(3, clt->fd,
		      "%s: found no alternative for requested '%s'\n",
		      __func__, dp500->requested_fullpath);
	    }
	  xfree(masq_path);
#endif
	}
      else
	{
	  char *real_fullpath;

	  /* Still nothing found -> scan direcortory tree for file.
	     This is used for subpictures, so they dont have to be
	     in the same directory as the main video file  */
	  real_fullpath = scan_directory_tree(dp500->subs_path, fullpath);
	  if (real_fullpath == NULL)
	    {
	      /* Nothing found in subdirs? -> status Not Found \ 404 */
	      status = 404;
	    }
	  else
	    {
	      /* Found in subdir -> status OK \ 200 */
	      xfree(dp500->requested_fullpath);
	      dp500->requested_fullpath = xstrdup(fullpath);
	      xfree(dp500->real_fullpath);
	      dp500->real_fullpath = real_fullpath;
	      do_log2(3, clt->fd,
		      "%s: found alternative '%s' for requested '%s'\n",
		      __func__, dp500->real_fullpath,
		      dp500->requested_fullpath);
	    }
	}
    }
  else
    {
      if (strcasecmp(fext, "avi") == 0)
	{
	  MainAVIHeader avih;

	  /* Parse avi header */
	  xfree(dp500->requested_fullpath);
	  dp500->requested_fullpath = xstrdup(fullpath);
	  xfree(dp500->real_fullpath);
	  dp500->real_fullpath = NULL;
	  get_avi_info(fullpath, &avih);
	}
      else
	{
	  do_log2(3, clt->fd, "%s: no special action requiered for '%s'\n",
		  __func__, fext);
	}
    }

out:
  /* Noticy playback device with status */
  snprintf(repl, sizeof(repl), "%i", status);
  write(clt->fd, repl, strlen(repl));
  do_log2(2, clt->fd, "%s: replied '%i'\n", __func__, status);

  /* Update Playlist(s) (only keep track of action1 for now) */
  if (status == 200 && clt->action == 1)
    {
      if (playlist_update_recent(fullpath) != 1)
	do_log(2, "%s: playlist_update_recent() failed\n", __func__);
    }

  do_log2(2, clt->fd, "%s: leaving, real '%s', requested '%s'\n", __func__,
	  dp500->real_fullpath, dp500->requested_fullpath);
}

/* EOF */
