/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: cmd_size.c,v 1.22 2005/09/12 11:08:43 eku Exp $
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include "dp500.h"
#include "logging.h"
#include "xmalloc.h"
#include "cmds.h"


void
cmd_size(char *args, t_client * clt, t_dp500 * dp500)
{
  char *p1, *p2;
  char *fullpath;
  off_t size = 0;
  char resp[16];

  p1 = strrchr(args, '|');
  p2 = strrchr(args, '\\');
  if (p1 == NULL)
    {
      do_log2(2, clt->fd, "%s: invalid request '%s'\n", __func__, args);
      return;
    }
  *p1 = '\0';
  if (p2 != NULL)
    *p2 = '/';

  fullpath = args;

  do_log2(4, clt->fd, "%s: path '%s'\n", __func__, fullpath);

  if (dp500->requested_fullpath != NULL)
    {
      do_log2(2, clt->fd, "%s: got '%s' (requested '%s', real '%s')\n",
	      __func__, fullpath, dp500->requested_fullpath,
	      dp500->real_fullpath);
    }

  if (dp500->real_fullpath != NULL &&
      strcmp(fullpath, dp500->requested_fullpath) == 0 &&
      strlen(dp500->real_fullpath) > 0)
    {
      do_log2(2, clt->fd, "%s: set path to real '%s' (%s)\n", __func__,
	      fullpath, dp500->real_fullpath);
      strncpy(fullpath, dp500->real_fullpath, sizeof(fullpath) - 1);
    }
  else
    {
      do_log2(2, clt->fd, "%s: set path to args '%s' (%s)\n", __func__,
	      fullpath, fullpath);
    }

  if (strncmp(fullpath, "http://", 7) == 0)
    {
      /* Size on a stream : return MAX */
      size = 1000 * 1000 * 1024;
    }
  else
    {
#ifdef VCDSUPPORT
      if (strcmp("avseq01.mpg", fullpath + strlen(fullpath) - 11) == 0)
	{
	  char *vcdimage;
	  char *command;
	  FILE *f;

	  /* Get size from vcd-info tool */
	  vcdimage = xstrdup(dp500->requested_fullpath);
	  strcpy(vcdimage + strlen(vcdimage) - 3, "bin");
	  command = xmalloc(100 + strlen(vcdimage));
	  do_log2(2, clt->fd, "%s: vcdimage name '%s'\n", __func__, vcdimage);

	  sprintf(command,
		  "vcd-info -b %s|sed -e'/AVSEQ01.MPG/!d;s/^.*] *\\([0-9]*\\) [^]]*$/\\1/g'",
		  vcdimage);
	  do_log2(2, clt->fd, "%s: vcdimage command '%s'\n", __func__,
		  command);

	  f = popen(command, "r");
	  if (f == NULL || feof(f))
	    {
	      do_log(1, "%s: popen(%s) failed: %s\n", __func__, command,
		     strerror(errno));
	    }
	  else
	    {
	      char *s = fgets(resp, sizeof(resp), f);
	      pclose(f);
	      if (s == NULL || strlen(resp) == 0)
		{
		  do_log(1, "%s: no output from '%s'\n", __func__, command);
		}
	      else
		{
		  size = strtol(resp, NULL, 10);
		}
	    }
	  do_log2(2, clt->fd, "%s: vcdimage size '%lu'\n", __func__,
		  (unsigned long)size);

	  xfree(command);
	  xfree(vcdimage);
	}
      else
#endif
	{
	  /* Size on a regular file */
	  struct stat st;

	  if (stat(fullpath, &st) != 0)
	    {
	      do_log(1, "%s: stat(%s) failed: %s\n", __func__, fullpath,
		     strerror(errno));
	    }
	  else
	    size = st.st_size;
	}
    }

  /* FIXME: filter out size when file is not readable yet (=1G ?) */
  if (size == 0 /*|| size > 2000000000U */ )
    {
      size = 2000000000U;
    }
  snprintf(resp, sizeof(resp), "%015llu", (unsigned long long)size);
  write(clt->fd, resp, 15);
  do_log2(2, clt->fd, "%s: '%s' - replied '%s'\n", __func__, fullpath, resp);
}

/* EOF */
