/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: config.c,v 1.23 2005/09/12 11:08:43 eku Exp $
 */

#include <sys/types.h>
#include <sys/param.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include "dp500.h"
#include "logging.h"
#include "xmalloc.h"
#include "config.h"


static int
config_read(const char *file, t_dp500 * dp500)
{
  FILE *f;

  /* Free all allocated memory in case of previously failed call */
  xfree(dp500->mp3_path);
  xfree(dp500->video_path);
  xfree(dp500->images_path);
  xfree(dp500->proxy_host);
  dp500->proxy_port = 0;
  xfree(dp500->proxy_user);
  xfree(dp500->proxy_password);
  xfree(dp500->playlist_path);
  xfree(dp500->subs_path);
  xfree(dp500->reboot_cmd);
  xfree(dp500->halt_cmd);
  xfree(dp500->playlist_gong_path);
  xfree(dp500->convert_win_m3u_from);
  xfree(dp500->convert_win_m3u_to);
  dp500->show_server_menu = 0;
  dp500->enable_playlists = 0;
  dp500->enable_kml_server = 0;

  f = fopen(file, "r");
  if (f)
    {
      char buf[1024];

      while (fgets(buf, sizeof(buf), f) != NULL)
	{
	  size_t len = strlen(buf);

	  *(buf + len - 1) = 0;	/* Kill newline */
	  if (len > strlen("MP3_PATH") && strncmp(buf, "MP3_PATH=", 9) == 0)
	    dp500->mp3_path = xstrdup(buf + 9);

	  else if (len > strlen("VIDEO_PATH") &&
		   strncmp(buf, "VIDEO_PATH=", 11) == 0)
	    dp500->video_path = xstrdup(buf + 11);

	  else if (len > strlen("IMAGES_PATH") &&
		   strncmp(buf, "IMAGES_PATH=", 12) == 0)
	    dp500->images_path = xstrdup(buf + 12);

	  else if (len > strlen("PROXY_HOST")
		   && strncmp(buf, "PROXY_HOST=", 11) == 0)
	    dp500->proxy_host = xstrdup(buf + 11);

	  else if (len > strlen("PROXY_PORT")
		   && strncmp(buf, "PROXY_PORT=", 11) == 0)
	    dp500->proxy_port = strtol(buf + 11, NULL, 10);

	  else if (len > strlen("PROXY_USER")
		   && strncmp(buf, "PROXY_USER=", 11) == 0)
	    dp500->proxy_user = xstrdup(buf + 11);

	  else if (len > strlen("PROXY_PASSWORD")
		   && strncmp(buf, "PROXY_PASSWORD=", 15) == 0)
	    dp500->proxy_password = xstrdup(buf + 15);

	  else if (len > strlen("DEBUG") && strncmp(buf, "DEBUG=", 6) == 0)
	    {
	      dp500->debug = strtol(buf + 6, NULL, 10);
	      if (dp500->debug > 4)
		dp500->debug = 4;
	    }
	  else if (len > strlen("PLAYLIST_PATH") &&
		   strncmp(buf, "PLAYLIST_PATH=", 14) == 0)
	    dp500->playlist_path = xstrdup(buf + 14);

	  else if (len > strlen("RECENLTY_HISTORY") &&
		   strncmp(buf, "RECENLTY_HISTORY=", 17) == 0)
	    dp500->playlist_recent_entries =
	      strtol(buf + 17, NULL, 10) - 1;

	  else if (len > strlen("SUBS_PATH") &&
		   strncmp(buf, "SUBS_PATH=", 10) == 0)
	    dp500->subs_path = xstrdup(buf + 10);

	  else if (len > strlen("REBOOT_CMD") &&
		   strncmp(buf, "REBOOT_CMD=", 11) == 0)
	    dp500->reboot_cmd = xstrdup(buf + 11);

	  else if (len > strlen("HALT_CMD") &&
		   strncmp(buf, "HALT_CMD=", 9) == 0)
	    dp500->halt_cmd = xstrdup(buf + 9);

	  else if (len > strlen("PLAYLIST_GONG_PATH") &&
		   strncmp(buf, "PLAYLIST_GONG_PATH=", 19) == 0)
	    dp500->playlist_gong_path = xstrdup(buf + 19);

	  else if (len > strlen("CONVERT_WIN_M3U_FROM") &&
		   strncmp(buf, "CONVERT_WIN_M3U_FROM=", 21) == 0)
	    dp500->convert_win_m3u_from = xstrdup(buf + 21);

	  else if (len > strlen("CONVERT_WIN_M3U_TO") &&
		   strncmp(buf, "CONVERT_WIN_M3U_TO=", 19) == 0)
	    dp500->convert_win_m3u_to = xstrdup(buf + 19);

	  else if (len > strlen("SHOW_SERVER_MENU") &&
		   strncmp(buf, "SHOW_SERVER_MENU=", 17) == 0)
	    {
	      char *server_menu = buf + 17;
	      if (server_menu[0] == '1' || server_menu[1] == '1')
		dp500->show_server_menu = 1;
	    }

	  else if (len > strlen("ENABLE_PLAYLISTS") &&
		   strncmp(buf, "ENABLE_PLAYLISTS=", 17) == 0)
	    {
	      char *enable_playlist = buf + 17;
	      if (enable_playlist[0] == '1' || enable_playlist[1] == '1')
		dp500->enable_playlists = 1;
	    }

	  else if (len > strlen("ENABLE_KML_SERVER") &&
		   strncmp(buf, "ENABLE_KML_SERVER=", 18) == 0)
	    {
	      char *kml_server = buf + 18;
	      if (kml_server[0] == '1' || kml_server[1] == '1')
		dp500->enable_kml_server = 1;
	    }
	}

      fclose(f);
      f = NULL;

      /* Handle empty playlist path */
      if (dp500->playlist_path == NULL)
	{
	  char *playlist_path = xmalloc(PATH_MAX);
	  char *home = getenv("HOME");
	  if (home == NULL)
	    {
	      do_log(0, "no HOME environment set\n");
	      goto out;
	    }
	  snprintf(playlist_path, PATH_MAX, "%s/.dp500serv/", home);
	  dp500->playlist_path = playlist_path;
	}

      /* Create playlists dir, just in case */
      if (mkdir(dp500->playlist_path, S_IRUSR | S_IWUSR | S_IXUSR) < 0 &&
	  errno != EEXIST)
	{
	  do_log(1, "%s: mkdir(%s) failed: %s\n", __func__,
		 dp500->playlist_path, strerror(errno));
	  goto out;
	}

      /* Handle empty subs path */
      if (dp500->subs_path == NULL)
	{
	  do_log(0, "SUBS_PATH subtitles is missing in '%s'\n", file);
	  goto out;
	}

      if (dp500->enable_playlists == 1)
	{
	  /* Handle empty Playlist-Gong path */
	  if (dp500->playlist_gong_path == NULL)
	    {
	      do_log(0, "PLAYLIST_GONG_PATH is missing in '%s'\n", file);
	      goto out;
	    }

	  /* Handle wrong path to playlist_gong.mp3 */
	  if (access(dp500->playlist_gong_path, R_OK) < 0)
	    {
	      do_log(0, "PLAYLIST_GONG_PATH is wrong\n");
	      goto out;
	    }
	}

      /* Both strings must be set or none! */
      if ((dp500->convert_win_m3u_to != NULL &&
	   dp500->convert_win_m3u_from == NULL) ||
	  (dp500->convert_win_m3u_to == NULL &&
	   dp500->convert_win_m3u_from != NULL))
	{
	  do_log(0, "Convert-Strings from Win to Unix must both be set\n");
	  goto out;
	}

      do_log(1, "PATH mp3           : %s\n", dp500->mp3_path);
      do_log(1, "PATH video         : %s\n", dp500->video_path);
      do_log(1, "PATH images        : %s\n", dp500->images_path);
      do_log(1, "PATH playlists     : %s\n", dp500->playlist_path);
      do_log(1, "PATH subtitles     : %s\n", dp500->subs_path);
      do_log(1, "PATH Playlist-Gong : %s\n", dp500->playlist_gong_path);
      do_log(1, "Change m3u Windows-Path from '%s' ",
	     dp500->convert_win_m3u_from);
      do_log(1, "to '%s'\n", dp500->convert_win_m3u_to);

      if (dp500->proxy_host)
	{
	  do_log(1, "HTTP proxy         : %s\n", dp500->proxy_host);
	  do_log(1, "HTTP port          : %hu\n", dp500->proxy_port);
	  if (dp500->proxy_user)
	    {
	      do_log(1, "HTTP user          : %s\n", dp500->proxy_user);
	      do_log(1, "HTTP password      : %s\n", dp500->proxy_password);
	    }
	}

      if (dp500->enable_playlists)
	do_log(1, "Playlists enabled\n");
      if (dp500->show_server_menu)
	do_log(1, "Server menu enabled\n");
      if (dp500->enable_kml_server)
	do_log(1, "KML server enabled\n");

      return 1;
    }

out:
  if (f)
    fclose(f);

  return 0;
}

int
read_config(const char *conffile, t_dp500 * dp500)
{
  char path[PATH_MAX];
  const char *home;

  if (conffile)
    {
      strncpy(path, conffile, sizeof(path) - 1);
      if (config_read(path, dp500) == 1)
	goto out;
    }

  home = getenv("HOME");
  if (home && strlen(home) > 0)
    {
      snprintf(path, sizeof(path), "%s/.dp500serv/dp500.conf", home);
      if (config_read(path, dp500) == 1)
	goto out;

      snprintf(path, sizeof(path), "%s/.dp500serv/dp500serv.conf", home);
      if (config_read(path, dp500) == 1)
	goto out;
    }

  snprintf(path, sizeof(path), "dp500.conf");
  if (config_read(path, dp500) == 0)
    {
      do_log(0, "failed to load config file\n");
      return -1;
    }

out:
  do_log(0, "loaded config file '%s'\n", path);
  return 0;
}

/* EOF */
