/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: dp500.h,v 1.32 2005/09/17 14:58:54 eku Exp $
 */

#ifndef DP500_H_
# define DP500_H_

#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <pthread.h>


# define FIRMWARE "2.8.0"


typedef struct s_client t_client;

typedef struct s_mp3stream
{
  pthread_t thread;
  int going;
  const char *url;
  t_client *clt;
  int sock;
  int prebuffering;
  int eof;
  int http_buffer_size;
  int http_prebuffer;
  int buffer_length;
  int prebuffer_length;
  char *buffer;
  int rd_index;
  int wr_index;
  uint64_t buffer_read;
  const char *icy_name;
  int icy_metaint;
  const char *icy_title;
} t_mp3stream;

struct s_client
{
  struct s_client *next;
  pthread_t thread;
  int going;
  struct sockaddr_in sa;	/* client's address */
  int fd;
  char *buf;			/* command receive buffer */
  int curcmd;			/* command format in receive buffer */
  int action;			/* type of current action (1 or 2) */
  const char *path_subp_previous;
  size_t chunk_size;		/* subpicture chunk size */
  t_mp3stream *stream;		/* Shoutcast stream */
};

typedef struct s_KMLclient
{
  struct s_KMLclient *next;
  pthread_t thread;
  int going;
  struct sockaddr_in sa;	/* client's address */
  int fd;
  char *buf;			/* command receive buffer */
  const char *player_id;
  const char *player_brand;
  const char *player_model;
  const char *player_language;
  const char *player_firmware;
} t_KMLclient;

typedef struct s_dp500
{
  /* command line options */
  const char *conffile;
  const char *logfile;
  int daemon_mode;

  /* config file options */
  const char *mp3_path;
  const char *video_path;
  const char *subs_path;
  const char *images_path;

  const char *proxy_host;
  unsigned short proxy_port;
  const char *proxy_user;
  const char *proxy_password;

  int enable_playlists;
  int playlist_recent_entries;
  const char *playlist_path;
  const char *playlist_gong_path;
  const char *convert_win_m3u_from;
  const char *convert_win_m3u_to;

  int enable_kml_server;

  int debug;
  int show_server_menu;
  const char *reboot_cmd;
  const char *halt_cmd;

  /* runtime vars */
  int going;

  t_client *clients;
  t_KMLclient *KMLclients;

  int fd_tcp_listen;
  int fd_udp_listen;
  int fd_kml_listen;

  const char *playlist_now;

  /* FIXME: should be client specific */
  char *requested_fullpath;
  char *real_fullpath;
} t_dp500;

extern t_dp500 *gl_dp500;

#endif

/* EOF */
