/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: dvd_tools.c,v 1.7 2005/08/18 06:31:22 eku Exp $
 */

#ifdef DVDSUPPORT

#undef _GNU_SOURCE
#include <sys/types.h>
#include <sys/param.h>
#include <ctype.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <pthread.h>
#include <errno.h>
#include "dp500.h"
#include "xmalloc.h"
#include "logging.h"
#include "playlists.h"
#include "kill_server.h"
#include "dvd_tools.h"


/* libdvdread includes */
#include <dvdread/dvd_reader.h>
#include <dvdread/ifo_types.h>
#include <dvdread/ifo_read.h>
#include <dvdread/nav_read.h>
#include <dvdread/nav_print.h>

/* The ISO 639 language codes.
 * Language names with * prefix are not spelled in their own language
 */
static struct
{
  const char descr[3];
  const char lang_long[20];
} lang_tbl[] =
{
  { "  ", "Not Specified" },
  { "aa", "*Afar" },
  { "ab", "*Abkhazian" },
  { "af", "*Afrikaans" },
  { "am", "*Amharic" },
  { "ar", "*Arabic" },
  { "as", "*Assamese" },
  { "ay", "*Aymara" },
  { "az", "*Azerbaijani" },
  { "ba", "*Bashkir" },
  { "be", "*Byelorussian" },
  { "bg", "*Bulgarian" },
  { "bh", "*Bihari" },
  { "bi", "*Bislama" },
  { "bn", "*Bengali; Bangla" },
  { "bo", "*Tibetan" },
  { "br", "*Breton" },
  { "ca", "*Catalan" },
  { "co", "*Corsican" },
  { "cs", "*Czech(Ceske)" },
  { "cy", "*Welsh" },
  { "da", "Dansk" },
  { "de", "Deutsch" },
  { "dz", "*Bhutani" },
  { "el", "*Greek" },
  { "en", "English" },
  { "eo", "*Esperanto" },
  { "es", "Espanol" },
  { "et", "*Estonian" },
  { "eu", "*Basque" },
  { "fa", "*Persian" },
  { "fi", "Suomi" },
  { "fj", "*Fiji" },
  { "fo", "*Faroese" },
  { "fr", "Francais" },
  { "fy", "*Frisian" },
  { "ga", "*Irish" },
  { "gd", "*Scots Gaelic" },
  { "gl", "*Galician" },
  { "gn", "*Guarani" },
  { "gu", "*Gujarati" },
  { "ha", "*Hausa" },
  { "he", "*Hebrew" },		/* formerly iw */
  { "hi", "*Hindi" },
  { "hr", "Hrvatski" },		/* Croatian */
  { "hu", "Magyar" },
  { "hy", "*Armenian" },
  { "ia", "*Interlingua" },
  { "id", "*Indonesian" },		/* formerly in */
  { "ie", "*Interlingue" },
  { "ik", "*Inupiak" },
  { "in", "*Indonesian" },		/* replaced by id */
  { "is", "Islenska" },
  { "it", "Italiano" },
  { "iu", "*Inuktitut" },
  { "iw", "*Hebrew" },		/* replaced by he */
  { "hi", "*Hindi" },
  { "hr", "Hrvatski" },		/* Croatian */
  { "hu", "Magyar" },
  { "hy", "*Armenian" },
  { "ia", "*Interlingua" },
  { "id", "*Indonesian" },		/* formerly in */
  { "ie", "*Interlingue" },
  { "ik", "*Inupiak" },
  { "in", "*Indonesian" },		/* replaced by id */
  { "is", "Islenska" },
  { "it", "Italiano" },
  { "iu", "*Inuktitut" },
  { "iw", "*Hebrew" },		/* replaced by he */
  { "ja", "*Japanese" },
  { "ji", "*Yiddish" },		/* replaced by yi */
  { "jw", "*Javanese" },
  { "ka", "*Georgian" },
  { "kk", "*Kazakh" },
  { "kl", "*Greenlandic" },
  { "km", "*Cambodian" },
  { "kn", "*Kannada" },
  { "ko", "*Korean" },
  { "ks", "*Kashmiri" },
  { "ku", "*Kurdish" },
  { "ky", "*Kirghiz" },
  { "la", "*Latin" },
  { "ln", "*Lingala" },
  { "lo", "*Laothian" },
  { "lt", "*Lithuanian" },
  { "lv", "*Latvian, Lettish" },
  { "mg", "*Malagasy" },
  { "mi", "*Maori" },
  { "mk", "*Macedonian" },
  { "ml", "*Malayalam" },
  { "mn", "*Mongolian" },
  { "mo", "*Moldavian" },
  { "mr", "*Marathi" },
  { "ms", "*Malay" },
  { "mt", "*Maltese" },
  { "my", "*Burmese" },
  { "na", "*Nauru" },
  { "ne", "*Nepali" },
  { "nl", "Nederlands" },
  { "no", "Norsk" },
  { "oc", "*Occitan" },
  { "om", "*(Afan) Oromo" },
  { "or", "*Oriya" },
  { "pa", "*Punjabi" },
  { "pl", "*Polish" },
  { "ps", "*Pashto, Pushto" },
  { "pt", "Portugues" },
  { "qu", "*Quechua" },
  { "rm", "*Rhaeto-Romance" },
  { "rn", "*Kirundi" },
  { "ro", "*Romanian" },
  { "ru", "*Russian" },
  { "rw", "*Kinyarwanda" },
  { "sa", "*Sanskrit" },
  { "sd", "*Sindhi" },
  { "sg", "*Sangho" },
  { "sh", "*Serbo-Croatian" },
  { "si", "*Sinhalese" },
  { "sk", "*Slovak" },
  { "sl", "*Slovenian" },
  { "sm", "*Samoan" },
  { "sn", "*Shona" },
  { "so", "*Somali" },
  { "sq", "*Albanian" },
  { "sr", "*Serbian" },
  { "ss", "*Siswati" },
  { "st", "*Sesotho" },
  { "su", "*Sundanese" },
  { "sv", "Svenska" },
  { "sw", "*Swahili" },
  { "ta", "*Tamil" },
  { "te", "*Telugu" },
  { "tg", "*Tajik" },
  { "th", "*Thai" },
  { "ti", "*Tigrinya" },
  { "tk", "*Turkmen" },
  { "tl", "*Tagalog" },
  { "tn", "*Setswana" },
  { "to", "*Tonga" },
  { "tr", "*Turkish" },
  { "ts", "*Tsonga" },
  { "tt", "*Tatar" },
  { "tw", "*Twi" },
  { "ug", "*Uighur" },
  { "uk", "*Ukrainian" },
  { "ur", "*Urdu" },
  { "uz", "*Uzbek" },
  { "vi", "*Vietnamese" },
  { "vo", "*Volapuk" },
  { "wo", "*Wolof" },
  { "xh", "*Xhosa" },
  { "yi", "*Yiddish" },		/* formerly ji */
  { "yo", "*Yoruba" },
  { "za", "*Zhuang" },
  { "zh", "*Chinese" },
  { "zu", "*Zulu" },
  { "\0", "" }
};

static dvd_config DVD;
pthread_mutex_t mutex_dvd = PTHREAD_MUTEX_INITIALIZER;	/* Used for lockinng the dvd extraction */
pthread_mutex_t mutex2 = PTHREAD_MUTEX_INITIALIZER;
unsigned char data[1024 * DVD_VIDEO_LB_LEN];
int titleid, chapid, pgc_id, len, start_cell, cur_cell;
unsigned int cur_pack;
int angle, ttn, pgn, next_cell;
dvd_reader_t *dvd;
dvd_file_t *title;
ifo_handle_t *vmg_file;
tt_srpt_t *tt_srpt;
ifo_handle_t *vts_file;
vts_ptt_srpt_t *vts_ptt_srpt;
pgc_t *cur_pgc;
#if 0
char *temp_file;
#endif
char *source_dir;
char *title_nr;
char *start;
char *end;
int rc;
int chunk = 0;
int bytes_written = 0;
int current_position = 0;


/* Full language naming out 2char name */
static const char *
fulllanguage(uint16_t descr)
{
  int i;
  for (i = 0; lang_tbl[i].lang_long[0]; i++)
    {
      if (!memcmp(&descr, &lang_tbl[i].descr, 2))
	{
	  return (char *)lang_tbl[i].lang_long;
	}
    }
  return lang_tbl[i].lang_long;
}


/* Amount of TitleSets \ Titles \ Chapters \ Angles on DVD */
int
detect_DVD_titles(char *_path, t_client * clt)
{
  ifo_handle_t *ifohandle_vmg;
  ifo_handle_t *ifohandle_vts;
  pgc_t *pgc;
  uint i, title_nr;
  int j = 0;
  int previous_chapter_sequence_nr;
  char string[256];
  char *path;
  char duration_str[16];
  char aspect_str[16];
  char audio_str[64];

  /* Open DVD files */
  path = xstrdup(_path);
  path = strcat(path, "/");
  DVD.DVDhandle = DVDOpen(path);
  /* Reset variables */
  DVD.DVD_Total_ChapterSequences = 0;

  /* Fetch amount of title sets */
  ifohandle_vmg = ifoOpen(DVD.DVDhandle, 0);
  if (ifohandle_vmg == NULL)
    {
      do_log(1, "Drive doesn't contain a valid DVD disk (s%)\n", path);
      return FALSE;
    }
  else
    {
      do_log(1, "Opened DVD at %s\n", path);
    }

  DVD.DVD_TitleSets = ifohandle_vmg->vmgi_mat->vmg_nr_of_title_sets;
  do_log(2, "DVD contains %i title sets\n", DVD.DVD_TitleSets);
  DVD.DVD_Total_Titles = ifohandle_vmg->tt_srpt->nr_of_srpts;

  /* Loop titles and parse the whereabouts of the stream chunks
     (titlesets, titles, chapters, angles) */
  for (i = 0; i < ifohandle_vmg->tt_srpt->nr_of_srpts; i++)
    {
      previous_chapter_sequence_nr = -2;
      DVD.VT_belongs_to_VTS[i] =
	ifohandle_vmg->tt_srpt->title[i].title_set_nr;
      DVD.VT_is_Title_of_VTS[i] =
	ifohandle_vmg->tt_srpt->title[i].vts_ttn - 1;
      /* Open VTS ifo */
      do_log(4, "Opening DVD VTS %i\n", DVD.VT_belongs_to_VTS[i]);
      ifohandle_vts = ifoOpen(DVD.DVDhandle, DVD.VT_belongs_to_VTS[i]);
      if (ifohandle_vts->vmgi_mat)
	{
	  do_log(1, "Opened a VMG ifo, VTS was exspected\n");
	}
      if (ifohandle_vts->vtsi_mat)
	{
	  DVD.VTS_Titles[DVD.VT_belongs_to_VTS[i] - 1] =
	    ifohandle_vts->vts_ptt_srpt->nr_of_srpts;
	  DVD.VT_Chapters[i] = ifohandle_vmg->tt_srpt->title[i].nr_of_ptts;
	  DVD.VT_Angles[i] = ifohandle_vmg->tt_srpt->title[i].nr_of_angles;
	  /* Aspect */
	  switch (ifohandle_vts->vtsi_mat->vts_video_attr.
		  display_aspect_ratio)
	    {
	    case 0:
	      sprintf(aspect_str, "_4:3");
	      break;
	    case 3:
	      sprintf(aspect_str, "_16:9");
	      break;
	    default:
	      sprintf(aspect_str, "_Unknown");
	      break;
	    }
	  /* Audio type */
	  sprintf(audio_str, "_%s",
		  fulllanguage((ifohandle_vts->vtsi_mat->vts_audio_attr[0].
				lang_code >> 8) +
			       ((ifohandle_vts->vtsi_mat->vts_audio_attr[0].
				 lang_code & 0xff) << 8)));

	  for (j = 0;
	       j <
	       ifohandle_vts->vts_ptt_srpt->title[DVD.VT_is_Title_of_VTS[i]].
	       nr_of_ptts; j++)
	    {
	      if (previous_chapter_sequence_nr !=
		  (ifohandle_vts->vts_ptt_srpt->
		   title[DVD.VT_is_Title_of_VTS[i]].ptt[j].pgcn - 1))
		{
		  /* New CS */
		  do_log(4, "Opening DVD chapter %i (ifohandle=%i)",
			 DVD.DVD_Total_ChapterSequences,
			 ifohandle_vts->vts_ptt_srpt->title[DVD.
							    VT_is_Title_of_VTS
							    [i]].ptt[j].
			 pgcn - 1);
		  title_nr = i;
		  DVD.DVD_Total_ChapterSequences++;
		  DVD.CS_belongs_to_VT[DVD.DVD_Total_ChapterSequences - 1] =
		    title_nr;
		  DVD.CS_starts_at_Chapter[DVD.DVD_Total_ChapterSequences -
					   1] = j + 1;
		  /* Fill chapters in previous PC */
		  if (DVD.DVD_Total_ChapterSequences > 1 && j > 0)
		    {
		      DVD.CS_Chapters[DVD.DVD_Total_ChapterSequences - 2] =
			ifohandle_vts->vts_ptt_srpt->title[DVD.
							   VT_is_Title_of_VTS
							   [i]].ptt[j -
								    1].pgn;
		      do_log(4, "Contains %i\n",
			     DVD.CS_Chapters[DVD.DVD_Total_ChapterSequences -
					     2]);
		    }
		  pgc =
		    ifohandle_vts->vts_pgcit->pgci_srp[ifohandle_vts->
						       vts_ptt_srpt->
						       title[DVD.
							     VT_is_Title_of_VTS
							     [i]].ptt[j].
						       pgcn - 1].pgc;
		  if (pgc != NULL)
		    {
		      sprintf(duration_str, "%02x:%02x:%02x",
			      pgc->playback_time.hour,
			      pgc->playback_time.minute,
			      pgc->playback_time.second);
		    }
		  do_log(4, " duration: %s, aspect: %s, audio:%s\n",
			 duration_str, aspect_str, audio_str);
		}
	      previous_chapter_sequence_nr =
		ifohandle_vts->vts_ptt_srpt->title[DVD.VT_is_Title_of_VTS[i]].
		ptt[j].pgcn - 1;
	    }
	  /* Send line to playback device */
	  sprintf(string, "DVDTITLE%03i_%s%s%s.MPG", i + 1, duration_str,
		  aspect_str, audio_str);
	  send_item(string, path, string);
	  /* Fill chapters in last PC */
	  DVD.CS_Chapters[DVD.DVD_Total_ChapterSequences - 1] =
	    ifohandle_vts->vts_ptt_srpt->title[DVD.VT_is_Title_of_VTS[i]].
	    ptt[j - 1].pgn;
	}
      else
	{
	  do_log(1, "Cant open VTS for titleset %i\n", i + 1);
	}
      ifoClose(ifohandle_vts);
    }

  /* Largest title */
  DVD.CS[DVD.CS_current] = TRUE;

  /* Clean & Exit */
  ifoClose(ifohandle_vmg);
  return TRUE;
}


/* Read a title from source and output the stream */
#if 0
static int
is_nav_pack(unsigned char *buffer)
{
  return (buffer[41] == 0xbf && buffer[1027] == 0xbf);
}
#endif


static void *
dvd_to_mpeg_init(void *path)
{
  /* Lock */
  rc = pthread_mutex_lock(&mutex2);
  do_log(4, "Inside DVD extraction thread, starting (pid=%i, path=%s)\n",
	 getpid(), (char *)path);

#if 0
  temp_file = xstrdup(path);
#endif
  source_dir = strndup(path, (size_t)((int)rindex(path, 47) - (int)path));
  do_log(4, "Inside DVD extraction thread, sourcedir = %s\n", source_dir);

  end = strstr(path, ".") - 10;
  start = strstr(path, "DVDTITLE") + 9;
  title_nr = strndup(start, end - start);
  do_log(4, "Inside DVD extraction thread, title_nr = %s\n", title_nr);

  titleid = atoi(title_nr) - 1;
  chapid = 0;			/* atoi( argv[ 3 ] ) - 1; */
  angle = 0;			/* atoi( argv[ 4 ] ) - 1; */

    /**
     * Open the disc.
     */
  dvd = DVDOpen(source_dir);
  if (!dvd)
    {
      do_log(1, "Couldn't open DVD: %s\n", source_dir);
      return NULL;
    }
  else
    do_log(4, "Opened DVD: %s\n", source_dir);
    /**
     * Load the video manager to find out the information about the titles on
     * this disc.
     */
  vmg_file = ifoOpen(dvd, 0);
  if (!vmg_file)
    {
      do_log(1, "Can't open VMG info.\n");
      DVDClose(dvd);
      return NULL;
    }
  else
    do_log(4, "Opened VMG info.\n");
  tt_srpt = vmg_file->tt_srpt;


    /**
     * Make sure our title number is valid.
     */
  do_log(2, "There are %d titles on this DVD.\n", tt_srpt->nr_of_srpts);
  if (titleid < 0 || titleid >= tt_srpt->nr_of_srpts)
    {
      do_log(1, "Invalid title %d.\n", titleid + 1);
      ifoClose(vmg_file);
      DVDClose(dvd);
      return NULL;
    }


    /**
     * Make sure the chapter number is valid for this title.
     */
  do_log(2, "There are %d chapters in this title.\n",
	 tt_srpt->title[titleid].nr_of_ptts);

  if (chapid < 0 || chapid >= tt_srpt->title[titleid].nr_of_ptts)
    {
      do_log(1, "Invalid chapter %d\n", chapid + 1);
      ifoClose(vmg_file);
      DVDClose(dvd);
      return NULL;
    }


    /**
     * Make sure the angle number is valid for this title.
     */
  do_log(2, "There are %d angles in this title.\n",
	 tt_srpt->title[titleid].nr_of_angles);
  if (angle < 0 || angle >= tt_srpt->title[titleid].nr_of_angles)
    {
      do_log(1, "Invalid angle %d\n", angle + 1);
      ifoClose(vmg_file);
      DVDClose(dvd);
      return NULL;
    }


    /**
     * Load the VTS information for the title set our title is in.
     */
  vts_file = ifoOpen(dvd, tt_srpt->title[titleid].title_set_nr);
  if (!vts_file)
    {
      do_log(1, "Can't open the title %d info file.\n",
	     tt_srpt->title[titleid].title_set_nr);
      ifoClose(vmg_file);
      DVDClose(dvd);
      return NULL;
    }


    /**
     * Determine which program chain we want to watch.  This is based on the
     * chapter number.
     */
  ttn = tt_srpt->title[titleid].vts_ttn;
  vts_ptt_srpt = vts_file->vts_ptt_srpt;
  pgc_id = vts_ptt_srpt->title[ttn - 1].ptt[chapid].pgcn;
  pgn = vts_ptt_srpt->title[ttn - 1].ptt[chapid].pgn;
  cur_pgc = vts_file->vts_pgcit->pgci_srp[pgc_id - 1].pgc;
  start_cell = cur_pgc->program_map[pgn - 1] - 1;


    /**
     * We've got enough info, time to open the title set data.
     */
  title = DVDOpenFile(dvd, tt_srpt->title[titleid].title_set_nr,
		      DVD_READ_TITLE_VOBS);
  if (!title)
    {
      do_log(1, "Can't open title VOBS (VTS_%02d_1.VOB).\n",
	     tt_srpt->title[titleid].title_set_nr);
      ifoClose(vts_file);
      ifoClose(vmg_file);
      DVDClose(dvd);
      return NULL;
    }

  /* Open temp file */
#if 0
  fd_temp = fopen(temp_file,"w");
#endif
  rc = pthread_mutex_unlock(&mutex2);

  next_cell = start_cell;

  /* Clean and return */
  return NULL;
}


/* Internal output buffer handler */
static int
dvd_add_to_buffer(char *buf_out, int buf_out_off, int buf_out_siz,
		  char *buf_in, size_t len)
{
  memcpy((char *)(buf_out + buf_out_off), buf_in, len);
  /* return new output buffer offset */
  return (buf_out_off + len);
}


int
dvd_to_mpeg_read(uint position, int len_requested, t_client * clt)
{
  uint bytes_written = 0;
  int ret;
  int done = 0;
  static int data_offset = -1;
  static int data_len = 0;
  static char *buffer = NULL;
  static int buffer_offset = 0;
  static int buffer_size = 0;
  static char *buffer_output = NULL;
  static int buffer_output_offset = 0;
  static int buffer_output_size = 0;

  /* Positioning */
  do_log(4,
	 "GET: DVD: Requested position: %u, internal position: %u, lenght: %i, fd-out: %i\n",
	 position, current_position, len_requested, clt->fd);
  if (position != current_position)
    {
      do_log(1, "Searching not yet implemented in DVD reading..\n");
      return 1;
    }

  /* Init buffer */
  if (buffer == NULL)
    {
      buffer = xmalloc(256000);	/* MPEG2 is requested in blocks of 131072 in FW 2.8.0 */
      buffer_size = 256000;
      buffer_offset = 0;
      buffer_output = xmalloc(256000);
      buffer_output_size = 256000;
      buffer_output_size = 0;
    }

  /* Writeout any data buffered from previous read */
  if (data_offset > 0)
    {
      if ((data_len - data_offset) > len_requested)
	{
	  do_log(4, "DVD WRITE(1): %li bytes to descriptor %i\n",
		 len_requested, clt->fd);
#if 0
	  ret = write(clt->fd, data+data_offset, len_requested);
#endif
	  buffer_output_offset =
	    dvd_add_to_buffer(buffer_output, buffer_output_offset,
			      buffer_output_size, data + data_offset,
			      len_requested);
	  data_offset += len_requested;
	  done = 1;
	}
      else
	{
	  do_log(4, "DVD WRITE(2): %li bytes to descriptor %i\n",
		 (data_len - data_offset), clt->fd);
#if 0
	  ret = write(clt->fd, data+data_offset, (data_len-data_offset) );
#endif
	  buffer_output_offset =
	    dvd_add_to_buffer(buffer_output, buffer_output_offset,
			      buffer_output_size, data + data_offset,
			      (data_len - data_offset));
	  done = 0;
	}
    }

    /**
     * Playback by cell in this pgc, starting at the cell for our chapter.
     */
  if (done == 0)
    for (cur_cell = start_cell; next_cell < cur_pgc->nr_of_cells;)
      {
	do_log(4, "DVD READ: fetching requested data..\n");
	data_offset = 0;
	cur_cell = next_cell;

	/* Check if we're entering an angle block. */
	if (cur_pgc->cell_playback[cur_cell].block_type
	    == BLOCK_TYPE_ANGLE_BLOCK)
	  {
	    int i;

	    cur_cell += angle;
	    for (i = 0;; ++i)
	      {
		if (cur_pgc->cell_playback[cur_cell + i].block_mode
		    == BLOCK_MODE_LAST_CELL)
		  {
		    next_cell = cur_cell + i + 1;
		    break;
		  }
	      }
	  }
	else
	  {
	    next_cell = cur_cell + 1;
	  }


	/**
         * We loop until we're out of this cell.
         */
	for (cur_pack = cur_pgc->cell_playback[cur_cell].first_sector;
	     cur_pack < cur_pgc->cell_playback[cur_cell].last_sector;)
	  {

	    dsi_t dsi_pack;
	    unsigned int next_vobu, next_ilvu_start, cur_output_size;


	    /**
             * Read NAV packet.
             */
	    len = DVDReadBlocks(title, (int)cur_pack, 1, data);
	    if (len != 1)
	      {
		do_log(1, "Read failed for block %d\n", cur_pack);
		ifoClose(vts_file);
		ifoClose(vmg_file);
		DVDCloseFile(title);
		DVDClose(dvd);
		return 1;
	      }
	    data_len = DVD_VIDEO_LB_LEN;

	    /* The KiSS needs the nav packs as well in order to look for ac52 audio.. ?? */
#if 0
	    fwrite(data, 1, DVD_VIDEO_LB_LEN, fd_temp );
#endif
	    if (len_requested > (bytes_written + DVD_VIDEO_LB_LEN))
	      {
		do_log(4, "DVD WRITE(3): %li bytes to descriptor %i\n",
		       DVD_VIDEO_LB_LEN, clt->fd);
#if 0
		ret = write(clt->fd, data, DVD_VIDEO_LB_LEN);
#endif
		buffer_output_offset =
		  dvd_add_to_buffer(buffer_output, buffer_output_offset,
				    buffer_output_size, data,
				    DVD_VIDEO_LB_LEN);
		bytes_written += DVD_VIDEO_LB_LEN;
		data_offset = -1;
	      }
	    else
	      {
		do_log(4, "DVD WRITE(4): %li bytes to descriptor %i\n",
		       (len_requested - bytes_written), clt->fd);
#if 0
		ret = write(clt->fd, data, (len_requested-bytes_written) );
#endif
		buffer_output_offset =
		  dvd_add_to_buffer(buffer_output, buffer_output_offset,
				    buffer_output_size, data,
				    (len_requested - bytes_written));
		bytes_written = len_requested;
		data_offset = len_requested;
		break;
	      }

	    /**
             * Parse the contained dsi packet.
             */
	    navRead_DSI(&dsi_pack, &data[DSI_START_BYTE]);
#if 0
	    assert( cur_pack == dsi_pack.dsi_gi.nv_pck_lbn );
#endif

	    /**
             * Determine where we go next.  These values are the ones we mostly
             * care about.
             */
	    next_ilvu_start = cur_pack
	      + dsi_pack.sml_agli.data[angle].address;
	    cur_output_size = dsi_pack.dsi_gi.vobu_ea;


	    /**
             * If we're not at the end of this cell, we can determine the next
             * VOBU to display using the VOBU_SRI information section of the
             * DSI.  Using this value correctly follows the current angle,
             * avoiding the doubled scenes in The Matrix, and makes our life
             * really happy.
             *
             * Otherwise, we set our next address past the end of this cell to
             * force the code above to go to the next cell in the program.
             */
	    if (dsi_pack.vobu_sri.next_vobu != SRI_END_OF_CELL)
	      {
		next_vobu = cur_pack
		  + (dsi_pack.vobu_sri.next_vobu & 0x7fffffff);
	      }
	    else
	      {
		next_vobu = cur_pack + cur_output_size + 1;
	      }

	    cur_pack++;

	    /**
             * Read in and output cursize packs.
             */
	    len = DVDReadBlocks(title, (int)cur_pack, cur_output_size, data);
	    if (len != (int)cur_output_size)
	      {
		do_log(1, "Read failed for %d blocks at %d\n",
		       cur_output_size, cur_pack);
		ifoClose(vts_file);
		ifoClose(vmg_file);
		DVDCloseFile(title);
		DVDClose(dvd);
		return 1;
	      }
	    data_len = DVD_VIDEO_LB_LEN * cur_output_size;

#if 0
	    fwrite( data, cur_output_size, DVD_VIDEO_LB_LEN, );
#endif
	    if (len_requested >
		(bytes_written + DVD_VIDEO_LB_LEN * cur_output_size))
	      {
		do_log(4, "DVD WRITE(5): %li bytes to descriptor %i\n",
		       cur_output_size * DVD_VIDEO_LB_LEN, clt->fd);
#if 0
		ret = write(clt->fd,data,cur_output_size*DVD_VIDEO_LB_LEN);
#endif
		buffer_output_offset =
		  dvd_add_to_buffer(buffer_output, buffer_output_offset,
				    buffer_output_size, data,
				    cur_output_size * DVD_VIDEO_LB_LEN);
		cur_pack = next_vobu;
		bytes_written += (cur_output_size * DVD_VIDEO_LB_LEN);
		data_offset = -1;
	      }
	    else
	      {
		do_log(4, "DVD WRITE(6): %li bytes to descriptor %i\n",
		       (len_requested - bytes_written), clt->fd);
#if 0
		ret = write(clt->fd,data, (len_requested-bytes_written) );
#endif
		buffer_output_offset =
		  dvd_add_to_buffer(buffer_output, buffer_output_offset,
				    buffer_output_size, data,
				    (len_requested - bytes_written));
		cur_pack = next_vobu;
		bytes_written = len_requested;
		data_offset = len_requested;
		break;
	      }


	    /* do we need to rollover to next output fill due to kiss's 2gb limit? */
#if 0
	       if (bytes_written>2000000000)
	       {
	       chunk += 1;
	       fclose(fd_temp);
	       xfree(temp_file);
	       temp_file = xmalloc(strlen(path)+10);
	       memset(temp_file,0,strlen(path)+10);
	       sprintf(temp_file,"%s_%i.mpg",(char*)path,chunk);
	       fd_temp = fopen(temp_file,"w");
	       bytes_written = 0;
	       }
#endif
	    if (bytes_written == len_requested)
	      {
		current_position = bytes_written;
		break;
	      }
	  }
      }

  /* Anough data collected to write to KiSS? */
  if (buffer_output_offset >= len_requested)
    {
      do_log(4, "GET: DVD: Writing %ibytes from BUFFER_OUT\n", len_requested);
      ret = write(clt->fd, buffer_output, len_requested);
      /* Move any data left to begin of buffer & reset offset */
      if (buffer_output_offset > len_requested)
	{
	  do_log(4,
		 "GET: DVD: Moving %ibytes in BUFFER_OUT for next request\n",
		 buffer_output_offset - len_requested);
	  memmove(buffer_output, (char *)(buffer_output + len_requested),
		  (buffer_output_offset - len_requested));
	  buffer_output_offset -= len_requested;
	}
      return 0;
    }
  else
    return 1;
}

#if 0
static void *
dvd_to_mpeg_close(uint position, int len)
{
  do_log(4, "Cleaning up for dvd_to_mpeg_thread\n");
  ifoClose(vts_file);
  ifoClose(vmg_file);
  DVDCloseFile(title);
  DVDClose(dvd);
#if 0
  fclose(fd_temp);
  xfree(temp_file);
#endif
  xfree(source_dir);
  do_log(4, "Exiting from dvd_to_mpeg_thread\n");
  pthread_exit((void *)0);	/* return temp_file; */
}
#endif

char *
dvd_to_mpeg_start(char *path)
{
  static char *path_temp_mpg = NULL;
  static pthread_attr_t attr;
  static pthread_t thread;
  int rc;


  do_log(4, "THREAD Setting up thread todo dvd extracting to %s\n", path);

  /* Are we free to go? */
  if (pthread_mutex_trylock(&mutex_dvd) == EBUSY)
    {
      /* Other extraction thread running: cancel it now */
      rc = pthread_mutex_unlock(&mutex_dvd);
    }

  /* Cleanup previous iterations */
  rc = pthread_cancel(thread);
  /* TODO cleanup temp files ...  unlink("./DVDTITLE*"); */

  /* init new pthread */
  path_temp_mpg = xmalloc(PATH_MAX);
  snprintf(path_temp_mpg, PATH_MAX, "%s/dvdseq01.mpg", getenv("PWD"));
  unlink(path_temp_mpg);
  xfree(path_temp_mpg);
  path_temp_mpg = xstrdup(path);
  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
  do_log(4, "Starting DVD extraction thread (path=%s)\n", path);
#if 0
  rc = pthread_create(&thread, &attr, dvd_to_mpeg_thread,(void*)path_temp_mpg);
  if (rc)
    {
      do_log(4, "ERROR; return code from pthread_create() is %d\n", rc);
      kill_server(rc);
    }
  else 
    do_log(4, "return code from pthread_create() is %d\n", rc);
#endif
  /* initialize DVD reading */
  dvd_to_mpeg_init(path_temp_mpg);

  path_temp_mpg = xmalloc(PATH_MAX);
  snprintf(path_temp_mpg, PATH_MAX, "%s/dvdseq01.mpg", getenv("PWD"));

  /* Give the extraction for time to start */
  rc = pthread_mutex_lock(&mutex2);
  rc = pthread_mutex_unlock(&mutex2);
  sleep(1);
#if 0
  rc = pthread_mutex_lock(&mutex_dvd);
  rc = pthread_mutex_unlock(&mutex_dvd);
#endif
  do_log(4, "returning from dvd_to_mpeg_start\n");
  return path;
}

#endif

/* EOF */
