/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: kill_server.c,v 1.13 2005/08/18 06:33:43 eku Exp $
 */

#include <stdio.h>
#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include "dp500.h"
#include "xmalloc.h"
#include "logging.h"
#include "clients.h"
#include "kml_server.h"
#include "kill_server.h"


void
kill_server(int exitcode)
{
  static pthread_mutex_t mutex_exit = PTHREAD_MUTEX_INITIALIZER;
  t_dp500 * dp500 = gl_dp500;
  int rc;

  do_log(4, "%s: entering\n", __func__);

  /* Lock */
  rc = pthread_mutex_lock(&mutex_exit);

#ifdef VCDSUPPORT
  /* Clean vcd temp files
     FIXME: kill VCD/DVD extraction threads */
  unlink("./avseq01.mpg");
  unlink("./avseq02.mpg");
  unlink("./videocd.xml");
#endif

  close(dp500->fd_tcp_listen);
  close(dp500->fd_udp_listen);
  close(dp500->fd_kml_listen);

  while ((volatile t_client *)dp500->clients != NULL)
    del_client(dp500->clients, dp500);

  while ((volatile t_KMLclient *)dp500->KMLclients != NULL)
    del_kml_client(dp500->KMLclients, dp500);

  xfree(dp500->mp3_path);
  xfree(dp500->video_path);
  xfree(dp500->images_path);
  xfree(dp500->proxy_host);
  xfree(dp500->proxy_user);
  xfree(dp500->proxy_password);
  xfree(dp500->playlist_path);
  xfree(dp500->subs_path);
  xfree(dp500->reboot_cmd);
  xfree(dp500->halt_cmd);
  xfree(dp500->playlist_gong_path);
  xfree(dp500->convert_win_m3u_from);
  xfree(dp500->convert_win_m3u_to);
  do_log(4, "%s: leaving\n", __func__);
  xfree(dp500);

  exit(exitcode);

  /* Unlock / dummy code */
  rc = pthread_mutex_unlock(&mutex_exit);
}

/* EOF */
