/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: listen.c,v 1.35 2005/09/15 08:21:42 eku Exp $
 */

#ifdef __alpha
# include <db.h>
#endif
#include <sys/types.h>
#include <sys/param.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include "dp500.h"
#include "logging.h"
#include "xmalloc.h"
#include "xlisten.h"
#include "clients.h"
#include "kml_server.h"
#include "kill_server.h"
#include "listen.h"


#define PORT_PCLINK	"8000"
#define PORT_KML	"8888"


static void
daemonize(void)
{
  pid_t child;

  switch (child = fork())
    {
    case -1:			/* failed */
      do_log(1, "%s: fork() failed: %s\n", __func__, strerror(errno));
      break;

    case 0:			/* child */
      waitpid(getppid(), NULL, 0);
      if (setsid() < 0)
	{
	  do_log(1, "%s: setsid() failed: %s\n", __func__, strerror(errno));
	  kill_server(1);
	}
      break;

    default:
      kill_server(0);
      break;
    }
}

static void
answer_client_searches_for_server(int listen_socket)
{
  struct sockaddr_in client;
  socklen_t clientLen;
  char str[INET_ADDRSTRLEN];	/* Kiss does only IPv4 */
  char msg[100];
  char hostname[MAXHOSTNAMELEN];

  /* Init buffer */
  memset(msg, 0x0, sizeof(msg));

  /* Receive message */
  clientLen = sizeof(client);
  if (recvfrom(listen_socket, msg, sizeof(msg), 0, (struct sockaddr *)&client,
	       &clientLen) < 0)
    {
      do_log(1, "%s: recvfrom() failed: %s\n", __func__, strerror(errno));
      return;
    }

  inet_ntop(AF_INET, &client.sin_addr, str, sizeof(str));

  /* Print received message */
  do_log(2, "SEARCH: received '%s' from %s:%hu/UDP\n",
	 msg, str, ntohs(client.sin_port));

  /* Did we got a kiss client request for server? */
  if (strcmp(msg, "ARE_YOU_KISS_PCLINK_SERVER?") != 0)
    return;

  usleep(120);			/* PC-Link 1.2 has a delay of 0.012 second */

  do_log(4, "SEARCH: received a request from Kiss client at %s\n", str);

  memset(hostname, 0, sizeof(hostname));
  gethostname(hostname, sizeof(hostname));
  if (strlen(hostname) > 0)
    {
      snprintf(msg, sizeof(msg), "%s %s on %s for firmware %s",
	       PACKAGE, VERSION, hostname, FIRMWARE);
    }
  else
    {
      snprintf(msg, sizeof(msg), "%s %s for firmware %s",
	       PACKAGE, VERSION, FIRMWARE);
    }

  /* Send data */
  if (sendto(listen_socket, msg, strlen(msg), 0,
	     (struct sockaddr *)&client, clientLen) < 0)
    {
      do_log(1, "%s: sendto() failed: %s\n", __func__, strerror(errno));
    }
  else
    {
      do_log(2, "SEARCH: replied '%s' back to Kiss client\n", msg);
    }
}

static void
accept_client(int listen_sock, t_dp500 * dp500)
{
  struct sockaddr_in client;
  socklen_t clientLen = sizeof(client);
  int sd;

  sd = accept(listen_sock, (struct sockaddr *)&client, &clientLen);
  if (sd != -1)
    {
      char str[INET_ADDRSTRLEN];	/* Kiss does only IPv4 */
      inet_ntop(AF_INET, &client.sin_addr, str, sizeof(str));

      do_log2(2, sd, "new connection from Kiss client at %s\n", str);
      add_new_client(&client, sd, dp500);
    }
}

static void
accept_kml_client(int listen_sock, t_dp500 * dp500)
{
  struct sockaddr_in client;
  socklen_t clientLen = sizeof(client);
  int sd;

  sd = accept(listen_sock, (struct sockaddr *)&client, &clientLen);
  if (sd != -1)
    {
      char str[INET_ADDRSTRLEN];	/* Kiss does only IPv4 */
      inet_ntop(AF_INET, &client.sin_addr, str, sizeof(str));

      do_log2(2, sd, "new KML connection from Kiss client at %s\n", str);
      add_new_kml_client(&client, sd, dp500);
    }
}

int
begin_listen(t_dp500 * dp500)
{
  int max;

  dp500->fd_tcp_listen = xlisten(NULL, PORT_PCLINK, SOCK_STREAM, NULL);
  if (dp500->fd_tcp_listen < 0)
    return -1;

  dp500->fd_udp_listen = xlisten(NULL, PORT_PCLINK, SOCK_DGRAM, NULL);
  if (dp500->fd_udp_listen < 0)
    return -1;

  max = MAX(dp500->fd_tcp_listen, dp500->fd_udp_listen);

  if (dp500->enable_kml_server)
    {
      dp500->fd_kml_listen = xlisten(NULL, PORT_KML, SOCK_STREAM, NULL);
      if (dp500->fd_kml_listen < 0)
	return -1;

      max = MAX(max, dp500->fd_kml_listen);
    }

  do_log(1, "%s %s for Kiss Firmware %s started\n",
	 PACKAGE, VERSION, FIRMWARE);

  if (dp500->daemon_mode)
    {
      do_log(0, "switching to daemon mode\n");
      daemonize();
    }

  while (dp500->going)
    {
      struct timeval tv;
      fd_set rfds;

      tv.tv_sec = 0;
      tv.tv_usec = 20000;
      FD_ZERO(&rfds);
      FD_SET(dp500->fd_tcp_listen, &rfds);
      FD_SET(dp500->fd_udp_listen, &rfds);
      if (dp500->enable_kml_server)
	FD_SET(dp500->fd_kml_listen, &rfds);

      if (select(max + 1, &rfds, NULL, NULL, &tv) > 0)
	{
	  if (FD_ISSET(dp500->fd_tcp_listen, &rfds))
	    {
	      accept_client(dp500->fd_tcp_listen, dp500);
	    }
	  else if (FD_ISSET(dp500->fd_udp_listen, &rfds))
	    {
	      answer_client_searches_for_server(dp500->fd_udp_listen);
	    }
	  else if (dp500->enable_kml_server &&
		   FD_ISSET(dp500->fd_kml_listen, &rfds))
	    {
	      accept_kml_client(dp500->fd_kml_listen, dp500);
	    }
	}
    }

  kill_server(0);
  /* NOTREACHED */ 
  return 0;
}

/* EOF */
