/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: logging.c,v 1.21 2005/08/20 15:31:16 eku Exp $
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <pthread.h>
#include <errno.h>
#include "dp500.h"
#include "logging.h"
#include "xmalloc.h"


void
do_log2(const int level, const int id, const char *text, ...)
{
  static pthread_mutex_t mutex_logging = PTHREAD_MUTEX_INITIALIZER;
  va_list ap;
  static int initialized = 0;
  static int fd;
  char buf[4096];
  size_t len;
  char time_str[64];
  time_t now = time(NULL);
  static int add_time_string = 1;
  int rc;

  /* Handle this log entry */
  if (gl_dp500->debug < level)
    {
      return;
    }

  /* Lock */
  rc = pthread_mutex_lock(&mutex_logging);

  /* Format current time to user displayable string */
  snprintf(time_str, sizeof(time_str), "%s", ctime(&now));
  time_str[strlen(time_str) - 1] = '\0';	/* remove newline */

  if (initialized == 0)
    {
      const char *home;
      const char *filename;

      if (gl_dp500->logfile == NULL)
	{
	  home = getenv("HOME");
	  if (home && strlen(home) > 0)
	    {
	      snprintf(buf, sizeof(buf), "%s/.dp500serv/", home);
	      filename = "dp500serv.log";
	    }
	  else
	    {
	      snprintf(buf, sizeof(buf), "/var/log/");
	      filename = "dp500serv";
	    }
	  if (mkdir(buf, S_IRUSR | S_IWUSR | S_IXUSR) < 0 && errno != EEXIST)
	    goto out;

	  strcat(buf, filename);
	}
      else
	{
	  strncpy(buf, gl_dp500->logfile, sizeof(buf) - 1);
	  buf[sizeof(buf) - 1] = '\0';
	}
      fd = open(buf, O_CREAT | O_APPEND | O_WRONLY, S_IREAD | S_IWRITE);
      if (fd < 0)
	goto out;

      if (gl_dp500->daemon_mode == 0)
	fprintf(stderr, "%s | created log file '%s'\n", time_str, buf);

      initialized = 1;
    }

  /* Parse formatted string */
  va_start(ap, text);
  vsnprintf(buf, sizeof(buf), text, ap);
  va_end(ap);
  len = strlen(buf);

  if (add_time_string == 1)
    {
      char id_str[8];

      if (id == -1)
	{
	  id_str[0] = '\0';
	}
      else
	{
	  snprintf(id_str, sizeof(id_str), "%4i | ", id);
	}

#ifdef LOG_TO_STDOUT
      if (gl_dp500->daemon_mode == 0)
	fprintf(stderr, "%s | %s%s", time_str, id_str, buf);
#endif
      write(fd, time_str, strlen(time_str));
      write(fd, " | ", 3);
      write(fd, id_str, strlen(id_str));
      write(fd, buf, len);

      /* Set display time to false.
         A newline on the end of buf will reactivate */
      add_time_string = 0;
    }
  else
    {
#ifdef LOG_TO_STDOUT
      if (gl_dp500->daemon_mode == 0)
	fprintf(stderr, buf);
#endif
      write(fd, buf, len);
    }

  /* Does current line has a newline? 
     If so, display time in front of next line, dont otherwise. */
  if ((add_time_string == 0) && (buf[len - 1] == '\n'))
    {
      add_time_string = 1;
    }

  /* Unlock */
out:
  rc = pthread_mutex_unlock(&mutex_logging);
}

/* EOF */
