/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: main.c,v 1.16 2005/09/15 08:21:42 eku Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <getopt.h>
#include "dp500.h"
#include "logging.h"
#include "xmalloc.h"
#include "config.h"
#include "listen.h"


t_dp500 *gl_dp500 = NULL;


static void
check_syntax(void)
{
  printf
    ("Usage: dp500serv [-d (daemon mode)] [-c configfile] [-l logfile]\n");
  exit(1);
}

static void
catch_signals(int sig)
{
  do_log(0, "got signal %i\n", sig);
  gl_dp500->going = 0;
}

int
main(int ac, char **av)
{
#ifdef HAVE_GETOPT_LONG
  static struct option long_options[] = {
    {"daemon", 0, 0, 'd'},
    {"config", 1, 0, 'c'},
    {"log", 1, 0, 'l'},
    {0, 0, 0, 0}
  };
  int option_index = 0;
#endif
  int c;
  struct sigaction sig;
  int rc;

  gl_dp500 = xmalloc(sizeof(t_dp500));
  gl_dp500->going = 1;

#ifdef HAVE_GETOPT_LONG
  while ((c = getopt_long(ac, av, "dc:l:", long_options, &option_index)) !=
	 EOF)
#else
  while ((c = getopt(ac, av, "dc:l:")) != EOF)
#endif
    {
      switch (c)
	{
	case 'd':
	  gl_dp500->daemon_mode = 1;
	  break;

	case 'c':
	  do_log(0, "using config file '%s'\n", optarg);
	  gl_dp500->conffile = optarg;
	  break;

	case 'l':
	  do_log(0, "using log file '%s'\n", optarg);
	  gl_dp500->logfile = optarg;
	  break;

	default:
	  check_syntax();
	  break;
	}
    }

  sig.sa_flags = 0;
  sig.sa_handler = catch_signals;
  sigemptyset(&sig.sa_mask);
  sigaction(SIGINT, &sig, 0);
  sigaction(SIGHUP, &sig, 0);

  rc = read_config(gl_dp500->conffile, gl_dp500);
  if (rc == 0)
    {
      rc = begin_listen(gl_dp500);
    }
  exit(rc);
}

/* EOF */
