/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: string_replace.c,v 1.3 2005/08/14 18:26:23 eku Exp $
 */

#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include "xmalloc.h"
#include "string_replace.h"


char *
string_replace(const char in[], const char what[], const char with[])
{

  char *str, *work, *back;
  const char *dp;
  int count;
  size_t size, lenwhat, lenwith, lenin;

  lenwhat = strlen(what);	/* Old substring */
  lenwith = strlen(with);	/* New substring */
  lenin = strlen(in);		/* Original string */

  /* Quit if nothing to do */
  if (what == NULL || in == NULL || with == NULL)
    return NULL;

  /* Count the number of replacements required */
  count = 0;
  for (dp = in; (dp = strstr(dp, what)) != NULL; dp += lenwhat)
    count++;

  /* Quit if nothing to do */
  if (count == 0)
    return NULL;

  /* Allocate enough space for original string plus expansions.
     Note that the working copy may initially be longer or shorter
     than the final result. */
  size = lenin + 1;
  if (lenwith > lenwhat)
    size += count * (lenwith - lenwhat);
  work = xmalloc(size);
  back = xmalloc(size);

  /* Do substitutions by splitting and rejoining strings */
  strncpy(work, in, size - 1);
  while ((str = strstr(work, what)) != NULL)
    {

      strcpy(back, str + lenwhat);
      work[str - work] = '\0';
      strcat(work, with);
      strcat(work, back);
    }

  /* Clean up and return new buffer */
  xfree(back);

  return work;
}

/* EOF */
