/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: treat_event.c,v 1.11 2005/08/17 07:46:15 eku Exp $
 */

#include <sys/types.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "dp500.h"
#include "logging.h"
#include "xmalloc.h"
#include "exec_cmd.h"
#include "clients.h"
#include "treat_event.h"


#define REC_LIMIT 512


static void
treat_cmd(t_client * clt, t_dp500 * dp500)
{
  char *cmd = clt->buf;
  clt->buf = 0;
  do_log2(2, clt->fd, "cmd '%s'\n", cmd);
  exec_cmd(cmd, clt, dp500);
  xfree(cmd);
}

static int
get_line_lim(t_client * clt)
{
  size_t len, i;
  ssize_t ret;

  len = clt->buf == NULL ? 0 : strlen(clt->buf);
  do_log2(4, clt->fd, "%s: len = %i, REC_LIMIT = %i\n", __func__, len,
	  REC_LIMIT);
  if (clt->buf)
    clt->buf = xrealloc(clt->buf, REC_LIMIT + len);
  else
    clt->buf = xmalloc(REC_LIMIT + len);

  if (len == 0)
    {
      ret = read(clt->fd, clt->buf + len, 3);
      if (ret != 3)
	return -1;
      *(clt->buf + 3) = 0;
      if (strcmp(clt->buf, "ACT") == 0)
	clt->curcmd = 0;
      else
	clt->curcmd = 1;
      len += 3;
    }

  if (clt->curcmd == 1)
    {
      /* Classic \r\n\r\n terminated command */
      for (i = 0; i < REC_LIMIT; ++i)
	{
	  ret = read(clt->fd, clt->buf + len + i, 1);
	  if (ret != 1)
	    return -1;
	  if (*(clt->buf + len + i) == '\r')
	    {
	      ret = read(clt->fd, clt->buf + len + i, 3);
	      *(clt->buf + len + i) = 0;
	      return 1;
	    }
	}
    }
  else
    {
      int numpipe = 0;

      /* ACTION | terminated command */
      for (i = 0; i < REC_LIMIT; ++i)
	{
	  if ((ret = read(clt->fd, clt->buf + len + i, 1)) == 0)
	    return -1;
	  if (ret != 1)
	    break;
	  if (*(clt->buf + len + i) == '|')
	    numpipe++;
	  if (numpipe == 1 && *(clt->buf + 7) == '2')
	    {
	      *(clt->buf + len + i) = 0;
	      return 1;
	    }
	  if (numpipe == 3 && *(clt->buf + 7) == '1')
	    {
	      *(clt->buf + len + i) = 0;
	      return 1;
	    }
	}
    }

  *(clt->buf + len + i - 1) = 0;
  return 0;
}

int
treat_event(t_client * clt, t_dp500 * dp500)
{
  int ret = get_line_lim(clt);
  if (ret == 1)
    treat_cmd(clt, dp500);
  else if (ret < 0)
    return -1;			/* Client has closed connection */

  return 0;
}

/* EOF */
