/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: xconnect.c,v 1.2 2005/08/14 18:26:23 eku Exp $
 */

#include <sys/types.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include "logging.h"
#include "xsocket.h"
#include "xconnect.h"


int
xconnect(const char *host, const char *serv)
{
  int sockfd, n;
  struct addrinfo hints, *res, *ressave;

  memset(&hints, 0, sizeof(hints));
  hints.ai_family = AF_UNSPEC;
  hints.ai_socktype = SOCK_STREAM;

  do_log(4, "resolving %s...\n", host);
  if ((n = getaddrinfo(host, serv, &hints, &res)) != 0)
    {
      do_log(1, "%s: getaddrinfo() failed: %s\n", __func__, gai_strerror(n));
      return -1;
    }

  ressave = res;

  do_log(2, "connecting...\n");
  do
    {
      sockfd = xsocket(res->ai_family, res->ai_socktype, res->ai_protocol);
      if (sockfd < 0)
	continue;		/* ignore this one */

      if (connect(sockfd, res->ai_addr, res->ai_addrlen) == 0)
	break;			/* success */

      close(sockfd);		/* ignore this one */
      sockfd = -1;
    }
  while ((res = res->ai_next) != NULL);

  if (res == NULL)		/* errno set from final connect */
    {
      do_log(1, "failed to connect to server '%s:%s'\n", host, serv);
    }

  freeaddrinfo(ressave);

  return sockfd;
}

/* EOF */
