/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: xlisten.c,v 1.5 2005/08/14 18:26:23 eku Exp $
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "logging.h"
#include "xsocket.h"
#include "xbind.h"
#include "xlisten.h"


int
xlisten(const char *host, const char *serv, int type, socklen_t * addrlenp)
{
  int listenfd, n;
  const int on = 1;
  struct addrinfo hints, *res, *ressave;

  memset(&hints, 0, sizeof(hints));
  hints.ai_flags = AI_PASSIVE;
  hints.ai_family = AF_INET;	/* Kiss does only IPv4 */
  hints.ai_socktype = type;

  if ((n = getaddrinfo(host, serv, &hints, &res)) != 0)
    {
      do_log(1, "%s: getaddrinfo() failed: %s\n", __func__, gai_strerror(n));
      return -1;
    }

  ressave = res;

  do
    {
      listenfd = xsocket(res->ai_family, res->ai_socktype, res->ai_protocol);
      if (listenfd < 0)
	continue;		/* ignore this one */

      if (setsockopt(listenfd, SOL_SOCKET, SO_REUSEADDR, (char *)&on,
		     sizeof(on)) == 0)
	{
	  if (xbind(listenfd, res->ai_addr, res->ai_addrlen) == 0)
	    {
	      if (type == SOCK_DGRAM)
		break;		/* everything done for a udp socket */

	      if (listen(listenfd, 10) == 0)
		break;		/* success */

	      do_log(1, "%s: listen() failed: %s\n", __func__,
		     strerror(errno));
	    }
	}

      close(listenfd);		/* ignore this one */
      listenfd = -1;
    }
  while ((res = res->ai_next) != NULL);

  if (res == NULL)		/* errno set from final connect */
    {
      do_log(1, "%s: failed to create a listen socket\n", __func__);
      goto out;
    }

  if (addrlenp)
    *addrlenp = res->ai_addrlen;

out:
  freeaddrinfo(ressave);

  return listenfd;
}

/* EOF */
