/*
 *  Kiss streaming server 
 *
 *  Copyright (C) 2003-2004 Thibault Lamy (xoubir@sourceforge.net)
 *                2003-2004 Jarl van Katwijk (bstard@sourceforge.net)
 *                2005      Erik Kunze (eku@sourceforge.net)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id: xmalloc.c,v 1.9 2005/08/14 18:26:23 eku Exp $
 */

#include <sys/types.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "logging.h"
#include "kill_server.h"
#include "xmalloc.h"


void *
xmalloc(size_t n)
{
  void *p = NULL;

  if (n == 0)
    {
      do_log(1, "%s: cannot malloc(0)\n", __func__);
      kill_server(1);
    }

  p = calloc(n, 1);
  if (p == 0)
    {
      do_log(1, "%s: malloc(%i) failed: %s\n", __func__, n, strerror(errno));
    }

  return p;
}

void *
xrealloc(void *p1, size_t n)
{
  void *p2 = realloc(p1, n);
  if (p2 == NULL)
    {
      do_log(1, "%s: realloc(%i) failed: %s\n", __func__, n, strerror(errno));
      kill_server(1);
    }
  return p2;
}

char *
xstrdup(const char *s1)
{
  char *s2 = xmalloc(strlen(s1) + 1);
  strcpy(s2, s1);
  return s2;
}

/* EOF */
